package SQL::Interpolate;

our $VERSION = "0.29";

use strict;
use base qw(Exporter);

our @EXPORT;
our %EXPORT_TAGS = (all => [qw(
    attr
    dbi_interp
    select_key
    limit
    make_sql_interp
    make_dbi_interp
    filter_sql
    sql_flatten
    sql_interp
)]);
our @EXPORT_OK = @{$EXPORT_TAGS{'all'}};

our $trace_sql = 0;
our $trace_filter = 0;

sub import
{
    my $class = shift;

    my $do_filter = 0; # source filtering option
    while(1) {
        if(defined($_[0]) && $_[0] eq 'FILTER') {
            shift;
            $do_filter = shift;
        }
        elsif(defined($_[0]) && $_[0] eq 'TRACE_SQL') {
            shift;
            $trace_sql = shift;
        }
        elsif(defined($_[0]) && $_[0] eq 'TRACE_FILTER') {
            shift;
            $trace_filter = shift;
        }
        else { last; }
     }
    unshift @_, $class;

    __PACKAGE__->export_to_level(1, @_);
    SQL::Interpolate::SQL->export_to_level(1);
    if($do_filter) {
        require SQL::Interpolate::Filter;
        goto &SQL::Interpolate::Filter::import;
    }
}

sub new
{
    my($class, %params) = @_;

    my $dbh = $params{dbh};
    my $relations = $params{relations};
    $relations = {} if ! defined $relations;
    my $self = bless {
        dbh       => $dbh,
        relations => $relations #FIX-copy?
    }, $class;
    return $self;
}

sub sql_flatten
{
    my(@parts) = @_;

    my $dbh = ref($parts[0]) eq 'DBI::db'          ? shift(@parts) :
              ref($parts[0]) eq 'SQL::Interpolate' ? shift(@parts)->{dbh} :
              undef;
    my $continue;
    do {
        $continue = 0;
        @parts = map {
            if(ref($_) eq 'SQL::Interpolate::SQL') {
                $continue = 1;
                @{$_->{list}}
            }
            elsif(UNIVERSAL::isa($_, 'SQL::Interpolate::Macro')) {
                $_->expand($dbh)
            } else { $_ }
        } @parts;
    }
    while($continue);

    return @parts;
}



sub sql_interp
{
    my $self;
    $self = $_[0] if ref $_[0] eq 'SQL::Interpolate';
    my @parts = sql_flatten @_;

    my $sql = '';
    my $id = qr/[a-zA-Z_\.]+/;
    my @bind;
    foreach my $part (@parts) {

        if(ref($part)) {
            if($sql =~ /\bIN\s*$/si) {
                $part = [ $$part ] if ref $part eq 'SCALAR';
                if(ref($part) eq 'ARRAY') {
                    if(@$part == 0) {
                        $sql =~ s/$id\s+IN\s*$/NULL/s or die 'ASSERT';
                    }
                    else {
                        push @bind, @$part;
                        $sql .= " (" . join(', ', map {'?'} @$part) . ")";
                    }
                }
                else {
                    die "Unrecognized '$part'.";
                }
            }
            elsif($sql =~ /\bSET\s*$/si && ref($part) eq 'HASH') {
                die 'ASSERT--num keys is zero' if keys %$part == 0;
                push @bind, values %$part;
                $sql .= " " . join(', ', map {"$_=?"} keys %$part);
            }
            elsif($sql =~ /\bINSERT\s+INTO\s*$id\s*$/si)
            {
                $part = [ $$part ] if ref $part eq 'SCALAR';
                if(ref($part) eq 'ARRAY') {
                    push @bind, @$part;
                    $sql .= " VALUES(" . join(', ', map {'?'} @$part) . ")";
                }
                elsif(ref($part) eq 'HASH') {
                    push @bind, values %$part;
                    $sql .= " (" . join(', ', keys %$part) . ")" .
                            " VALUES(" . join(', ', map {'?'} keys %$part) . ")";
                 }
                else { die "Unrecognized '$part'."; }
            }
            elsif(ref($part) eq 'SCALAR') {
                push @bind, $$part;
                $sql .= '?';
            }
            #FIX:document
            elsif(ref($part) eq 'HASH') # e.g. WHERE {x = 3, y = 4}
            {
                die if keys %$part == 0;
                push @bind, values %$part;
                my $s = join ' AND ', map {"$_=?"} keys %$part;
                $s = "($s)" if keys %$part > 1;
                $s = " $s";
                $sql .= $s;
            }
            else { die "Unrecognized '$part'."; }
        }
        else {
            $sql .= " $part";
        }
    }
    $sql =~ s{^ }{};
    $sql = $self->filter_sql($sql) if defined $self;
    print STDERR "DEBUG:interp[sql=$sql,bind=".join(':',@bind)."]\n"
        if $trace_sql;
    return($sql, @bind);
}

sub dbi_interp
{
    my $key;
    my $attr;
    my @args = grep {
        my $save = 1;
        if(ref($_) eq 'SQL::Interpolate::Key') {
            $key = $_; $save = 0;
        }
        elsif(ref($_) eq 'SQL::Interpolate::Attr') {
            $attr = {%$_}; $save = 0;
        }
        $save;
    } @_;
    my($sql, @bind) = &sql_interp(@args);
    my @params = ($sql);
    push @params, $$key if defined $key;
    push @params, $attr, @bind;
    return @params;
}


sub select_key
{
    my $key = shift;
    return bless \$key, "SQL::Interpolate::Key";
}

sub attr
{
    return bless {@_}, "SQL::Interpolate::Attr";
}

sub limit
{
    return SQL::Interpolate::Limit->new(@_);
}

sub sql
{
    return SQL::Interpolate::SQL->new(@_);
}


sub make_sql_interp
{
    my(@params) = @_;

    my $interp = sub {
        return sql_interp(@params, @_);
    };
    return $interp;
}

sub make_dbi_interp
{
    my(@params) = @_;

    my $interp = sub {
        return dbi_interp(@params, @_);
    };
    return $interp;
}

sub filter_sql
{
    my($self, $sql) = @_;

    #IMPROVE: possibly cache results

    # process LINK(...) macros.
    $sql =~ s{ \b LINK \( (.*?) \)
    }{
        my $params = $1;
        my @params = split /,/, $params;
        my $good = 1;
        my $last;
        for my $param (@params) {
            $param =~ s{^\s*|\s*$}{}gs;
            my $match = 0;
            done_param:
            for my $relation (values %{$self->{relations}}) {
                my $name_re = $relation->{name};
                if($param =~ /^($name_re)/gs) {
                    my($name, $name1, $name2) = ($1, $2, $3);
                    $param = [$name, [$name1, defined($name2) ? $name2 : ()], $relation];
                    $match = 1;
                    last done_param;
                       }
            }
            if(!$match) {
                die "Invalid param [$param] in LINK($params) macro in SQL template.";
            }
        }


        # relations touching entities.
        my %links;
        my @sql_snips;
        for my $param (@params) {

            for my $entity (@{$param->[1]}) {
                if(defined $links{$entity}) {
                    #print Dumper($entity, $links{$entity}, $param), "\n";
                    push @sql_snips,
                        $self->_single_link_sql($links{$entity}, $param);
                }
            }
            $links{$param->[1]->[0]} = $param;
            $links{$param->[1]->[1]} = $param if defined $param->[1]->[1];
        }

        my $sql = join ' AND ', @sql_snips;

        $sql = "($sql)" if @sql_snips > 1;
        $sql;
    }xge;

    my %keys;

    # process R(...) macros
    $sql =~ s{ \b R \( (.*?) \) }
    {
        my $name = $1;
        my $out;
        for my $relation_name (keys %{$self->{relations}}) {
            my $relation = $self->{relations}->{$relation_name};
            my $name_re = $relation->{name};
            if($name =~ /^($name_re)$/s) {
                my($name, $name1, $name2) = ($1, $2, $3);

                $keys{$name1} = "$name.$relation->{key}->[0]";
                $keys{$name2} = "$name.$relation->{key}->[1]"
                    if defined $relation->{key}->[1];
                $out = "$relation_name as $name";
                last;
                   }
        }
        if(! defined $out) {
            die "Unrecognized relation R($name) in SQL template [$sql].";
        }
        $out;
    }xge;

    while(my($name, $key) = each %keys) {
        $sql =~ s{ (?<!as\ ) \b \Q$name\E \b (?! \.) }{$key}xge;
    }

    #die Dumper($sql);
    return $sql;

}

# [private]
# Given instances of two relations, generate the SQL to link them.
# For example,
#   ['Sp', ['S', 'p'], $sales_ord_line] and
#   ['p', ['p'], $part]
# gives "Sp.part_nbr = p.part_nbr".
# params:
#   $e1 - entity 1
#   $e2 - entity 2
# where each instance is a arrayref of an entity relation name, a arrayref of
#   names of contained entities, and a relation
#   specification (as passed into C<relations>).
sub _single_link_sql
{
    my($self, $e1, $e2) = @_;
#    die Data::Dumper::Dumper($e1, $e2);
    my($left_idx, $right_idx);
    if   ($e1->[1]->[0] eq  $e2->[1]->[0])
        { $left_idx=0; $right_idx=0; }
    elsif(defined($e2->[1]->[1]) && $e1->[1]->[0] eq  $e2->[1]->[1])
        { $left_idx=0; $right_idx=1; }
    elsif(defined($e1->[1]->[1]) && $e1->[1]->[1] eq  $e2->[1]->[0])
        { $left_idx=1; $right_idx=0; }
    elsif(defined($e1->[1]->[1]) && defined($e2->[1]->[1]) &&
              $e1->[1]->[1] eq  $e2->[1]->[1])
        { $left_idx=1; $right_idx=1; }
    else {
        die "Invalid SQL link [$e1->[0] to $e2->[0]]";
    }

    my $sql = "$e1->[0].$e1->[2]->{key}->[$left_idx ]" . " = " .
              "$e2->[0].$e2->[2]->{key}->[$right_idx]";

    return $sql;
}

sub relations
{
    my($self, $relations) = @_;

    $self->{relations} = $relations; #FIX--copy?
}


package SQL::Interpolate::Macro;
# Macros embedded in the SQL (e.g. LIMIT macro).

package SQL::Interpolate::Limit;
our @ISA = qw(SQL::Interpolate::Macro);

# LIMIT macro.
# Note: we deter formatting until sql_interp since
# this function does not know anything about which
# SQL dialect to general SQL for.

sub new
{
    my($class, %params) = @_;

    my $self = bless {
        start => $params{start}, count => $params{count}
    }, $class;
    return $self;
}

sub expand
{
    my($self, $dbh) = @_;
    # FIX: generalize this and make more robust.
    #   Use SQL::AnyDBD.
    if(defined($dbh) && $dbh->{Driver}->{Name} eq 'mysql') {
        my $count = $self->{count};
        $count = "18446744073709551615" if ! defined $count; #inf
        my $start = $self->{start};
        $start = 0 if ! defined $start;
        die "ASSERT:$start" if $start !~ /^\d+$/s;
        die "ASSERT:$count" if $count !~ /^\d+$/s;
        return ("LIMIT $start, $count");
        # Note: We no longer use bind values here.
        # To use bind values, we might need to bind params to
	# integers to prevent quotes:
	#   $sth->bind_param(n, $val, SQL_INTEGER)
	# See e.g. http://www.mail-archive.com/dbi-users@perl.org/msg17815.html
	# It is not clear how this would be incorporated into dbi_interp.
	# It can probably be done with DBIx::Interpolate, but it may not
	# be worth it.
        #return ('LIMIT', \$self->{start}, ',', \$self->{count});
    }
    else {
        #FIX-what if values are undef?
        return ('LIMIT', \$self->{count}, 'OFFSET', \$self->{start});
    }
}

package SQL::Interpolate::SQL;
# Generated by the sql// operator when source filtering is enabled.

use strict;
use base qw(Exporter);
use overload '.'  => \&concat,
             '""' => \&stringify;

sub new
{
    my($class,
       @list   # list of SQL string, variable references,
               #   macro objects, and SQL objects.
    ) = @_;

    # Note that sql[INSERT INTO mytable $x] gets translated to
    #   q[INSERT INTO mytable], \$x 
    # regardless whether $x is a scalar or reference since it
    # would be difficult to know at source filtering time whether
    # $x is already a reference.  Therefore, we dereference any
    # double reference here (at run-time).
    do { $_ = $$_ if ref($_) eq 'REF' } for @list;

    my $self = {
        list => \@list
    };
    bless $self, $class;
    return $self;
}

# Concatenate SQL object with another expression.
# An SQL object can be concatenated with another SQL object,
# variable reference, or an SQL string.
sub concat
{
    my($a, $b, $inverted) = @_;

    my @params = ( @{$a->{list}},
                   ref($b) eq __PACKAGE__ ? @{$b->{list}} : ($b) );
    ($params[0], $params[1]) = ($params[1], $params[0]) if $inverted;
    my $c = new SQL::Interpolate::SQL(@params);

    return $c;
}

sub stringify
{
    my($a) = @_;
    return $a;
}

1;

__END__

=head1 NAME

SQL::Interpolate - Simplified interpolation of Perl variables into SQL
                   statements.

=head1 SYNOPSIS

=head2 Interpolating variables into SQL (sql_interp)

  use SQL::Interpolate qw(:all);
  
  my @colors = ('blue', 'green');
  my($sql, @bind) = sql_interp
      q[SELECT * FROM mytable WHERE color IN], \@colors,
      q[AND y =], \$x, q[OR], {z => 3, w => 2}
  );
  # Sets
  #   $sql = "SELECT * FROM mytable WHERE color IN (?, ?) " .
  #          "AND y = ? OR (z = ? AND w = ?)";
  #   @bind = ('blue', 'green', $x, 3, 2);
  # e.g. $dbh->selectall_arrayref($sql, undef, @bind);

=head2 Passing the result to DBI with the DBI adapter (dbi_interp)

  use DBI;
  
  $dbh->selectall_arrayref(dbi_interp
      q[SELECT * FROM mytable WHERE color IN], \@colors,
      q[AND y =], \$x, q[OR], {z => 3, w => 2}
  );
  # note: dbi_interp typically returns ($sql, \%attr, @bind)

=head2 Using DBIx::Interpolate to transparently use the DBI adapter

  use DBIx::Interpolate qw(:all);
  
  my $dbx = new DBIx::Interpolate(dbh => $dbh);
  $dbx->selectall_arrayref(
      q[SELECT * FROM table WHERE color IN], \@colors,
      q[AND y =], \$x
  );

=head2 Enabling source filtering (where it really gets interesting)

  use SQL::Interpolate FILTER => 1, qw(:all);
  
  $dbx->selectall_arrayref(sql[
      SELECT * FROM mytable WHERE color IN @colors
      AND y = $x OR {z => 3, w => 2}
  ]);

=head2 Various SQL examples

  my @colors = ('blue', 'green');
  my $rows = $dbh->selectall_hashref(dbi_interp
      qq[SELECT * FROM table WHERE color IN], \@colors,
      qq[AND y = ], \$x,
      limit(start => 1, count => 10),
      select_key('id')
  );
  # Equivalent to (assuming Postgres LIMIT syntax)
  #   my $rows = $dbh->selectall_hashref(qq[
  #       SELECT * FROM table WHERE color IN (?, ?) AND y = ?
  #       LIMIT ? OFFSET ?], 'id', undef, @colors, $x, 10, 1
  #   );

  $dbh->do(dbi_interp
      "INSERT INTO table", {color => $new_color,
                            shape => $new_shape}
  );
  # Equivalent to
  #   $dbh->do(qq[
  #       INSERT INTO table (color, shape) VALUES(?, ?)],
  #       undef, $new_color, $new_shape
  #   );

  $dbh->do(dbi_interp
      qq[UPDATE table SET], {
          color => $new_color,
          shape => $new_shape
      }, q[WHERE color <> ], \$color
  );
  # Equivalent to
  #   $dbh->do(qq[
  #       UPDATE table SET color = ?, shape = ?
  #       WHERE color <> ?], undef, $new_color, $new_shape
  #   );

=head2 Automating table joining

 $dbx->interp()->relations(...);
 $dbx->selectall_arrayref(qq[
     SELECT * FROM R(S), R(Sp), R(p)
     WHERE LINK(S,Sp,p) AND S = 123
 ]);
 # Generates SQL
 #   SELECT *
 #   FROM  sales_order as S, sales_order_line as Sp, part as p
 #   WHERE S.so_nbr = Sp.so_nbr AND Sp.part_nbr = p.part_nbr AND
 #         S.so_nbr = 123
  

=head1 DESCRIPTION

=head2 Purpose

This module provides convenient methods for interpolating Perl
variables into SQL statements.  To I<interpolate> means to "insert
between other things or parts" (Webster).  In the programming world, a
familiar example is string interpolation, of inserting variables into
strings:

  my $s = "The value of x is $x.";        # Perl
  sprintf(s, "This value of x is %d", x); # C

However, due to handling of special characters, variables cannot
always be interpolated verbatim to achieve correct results but must
instead be escaped according to rules dependent upon the application:

  medium  escape method
  ------  ---------------
  SQL     $dbh->quote(...) or bind values "?"
  regex   quotemeta(...) or \Q...\E
  HTML    CGI::escapeHTML(...)
  URL     URI::uri_escape(...)

When using DBI, one traditionally interpolates using bind values:

  $dbh->selectall_arrayref(
      "SELECT * FROM mytable WHERE x = ? AND y = ?", undef, $x, $y);

which can become unwieldy:

  $dbh->do(qq(
      INSERT INTO table (color, shape, width, height, length)
                  VALUES(?,     ?,     ?,     ?,      ?     )
  ), undef, $new_color, $new_shape, $width, $height, $length);

SQL::Interpolate enables you to achieve the same effect in a more
Perl-like manner:

  $dbx->do(sql[
      INSERT INTO table {
          color  => $new_color,
          shape  => $new_shape
          width  => $width,
          height => $height,
          length => $length
      }
  ]);

SQL::Interpolate's inline approach eliminates the need for various SQL
building techniques.  Rather than

  my $where = 'suid IN (' . join(',', map {'?'} @suid) . ')';
  $where = '1' if @suid == 0;
  my $rows = $dbh->selectall_arrayref(qq(
      SELECT thid, date_modified, title, suid, state
      FROM threads
      WHERE state <> ? AND $where
      ORDER BY t.date_modified DESC
      LIMIT ? OFFSET ?
  ), undef, @suid, $state, $count, $page*$count);

one achieves the same effect and use of bind values with

  my $rows = $dbx->selectall_arrayref(sql[
      SELECT thid, date_modified, title, suid, state
      FROM threads
      WHERE state <> $state AND suid IN @suids
      ORDER BY t.date_modified DESC
      LIMIT(start => $page*$count, count => $count)
  ]);

As shown in the SYNOPSIS, there's also more than one way to do it.
The choice of method depending on how much interpolation should be
integrated with DBI and whether source filtering should be enabled to
further streamline the syntax.  For example, with source filtering
disbled, the above is written using a list of items to be
interpolated:

  my $rows = $dbx->selectall_arrayref(q[
      SELECT thid, date_modified, suid, title, state, num_messages
      FROM threads
      WHERE state <> ], \$state, q[AND suid IN], \@suids, q[
      ORDER BY t.date_modified DESC],
      limit(start => $page*$count, count => $count)
  );

or you may obtain the SQL and bind values directly:

  my($sql, @bind) = sql_interp q[
      SELECT thid, date_modified, suid, title, state, num_messages
      FROM threads
      WHERE state <> ], \$state, q[AND suid IN], \@suids, q[
      ORDER BY t.date_modified DESC],
      limit(start => $page*$count, count => $count)
  ;

A secondary purpose of SQL::Interpolate is to provide macros
that both simplify your SQL and make it more portable.  The
LIMIT(...) macro or limit(...) function above can get transformed
into whichever LIMIT syntax your database supports:

  LIMIT ? OFFSET ?  # Postgres
  LIMIT ?, ?        # MySQL

provided you give SQL::Interpolate a database handle:

  $interp = new SQL::Interpolate(dbh => %dbh);
  ($sql, @bind) = $interp->sql_interp(...);

There is also a syntax that eliminates redundancy in table joins
as shown in the "Automatic table joining" section above.

The purpose of SQL::Interpolate, therefore, is to make SQL statements
in Perl more natural, less redundant, and less error-prone.
SQL::Interpolate takes your query specification and generates
correctly formatted SQL statements I<with bind values>.  These results
can then be passed into DBI or used for another purpose.

=head2 Comparison to other modules

SQL::Interpolate provides a convenient method of forming SQL queries
possibly parameterized by Perl variables.  Therefore, the purpose of
SQL::Interpolate is similar to that of SQL::Abstract except that
SQL::Interpolate gives the author more direct access to the underlying
SQL, and queries are expressed more in SQL than in OO method calls.
The SQL::Interpolate approach can be advantageous, at least in certain
cases, as it allows you to use the full expressivity of your database
query language while also providing a few macros that can be used to
increase the portability of the generated SQL.

The related DBIx::Interpolate module does nothing more than allow
certain DBI methods to accept an SQL::Interpolate::dbi_interp()-like
parameter list rather than the traditional ($statement, \%attr,
@bind_values)-like parameter list.

There are other datarbase modules such as Juerd Waalboer's
DBIx::Simple that facilitate converting database query results back
into Perl variables. Like SQL::Abstract, SQL::Interpolate and (by
implication) DBIx::Interpolate currently add nothing in this
regard. Since DBIx::Simple optionally interfaces with
SQL::Abstract, the possibility may exist of extending DBIx::Simple to
similarly interface with SQL::Interpolate. See also Tony
Bowden's Class::DBI for a more object-relational approach to database
access.

=head2 Theory

The basic concept of SQL::Interpolate is that one passes a list of
items to interpolate, and SQL::Interpolate generates an SQL statements
with bind values and the list of actual bing values.

  my($sql, @bind) = sql_interp @interp_list;           # functional

@interp_list can consist of

 -strings containing SQL
 -references to variables to be interpolated
  (scalarrefs, arrayrefs, and hasherfs)
 -macro objects (derived from SQL::Interpolate::Macro),
 -SQL::Interpolate::SQL objects (generated by sql// constructs
  when source filtering is enabled)

Additionally, the first element may be a database handle.  If
dbi_interp is used instead, some additional DBI-specific elements may
be passed.

Generally, the transformation is as follows.

Strings are appended to the $sql string largely verbatim except
to add additional whitespace and expand any macros.

Variable references are converted to placeholders
"?" in the generated SQL, and the contents of the references are
pushed onto the @bind array.

Other objects are typically converted into SQL strings and/or bindings.
dbi_interp can convert some DBI-specific objects into additional return
values expected by DBI methods (e.g. selectall_hashref):

  my(sql, $select_key, $attr, @bind) = dbi_interp @interp_list;

There is also an OO-interface, if you'd like to configure the interpolation:

  my $interp = new SQL::Interplolate(dbh => $dbh);
  my($sql, @bind) = $interp->sql_interp(@interp_list); # OO

SQL::Interpolate has some additional smarts to typically Do the Right
Thing in a context-sensitive way.  Refer to the C<sql_interp>
documentation for details.

SQL::Interpolate extends these basic concepts with source filtering
and improved DBI interoperability.  However, these extensions
are really just syntactic sugar.

=head2 Security notes

SQL::Interpolate is designed to properly escape or bind variables.
One sometimes sees people doing very unsafe things like this:

  use CGI;
  use DBI;
  ...
  my $q = new CGI;
  my $name = $q->param('name');
  $dbh->do("UPDATE mytable SET y = 0 WHERE x = '$name'");

The problem is that a malicious user could pass raw SQL into
$name (e.g. "one' or x = x or x ='")
so that your SQL statement becomes

  $dbh->do("UPDATE mytable SET y = 0 WHERE x = 'one' or x = x or x=''");

or worse.  It is not the intent nor a good idea to allow random web
users to execute arbitrary SQL on your database.

However, the SQL::Interpolate sql// string-like operator
does safely interpolate, so that

  $dbx->do(sql[UPDATE mytable SET y = 0 WHERE x = $name]);

or

  $dbx->do(sql[UPDATE mytable SET y = 0 WHERE x = ] . \$name);

(both using DBIx::Interpolate) is not the same thing as

  $dbh->do('UPDATE mytable SET y = 0 WHERE x = ' . $name);

but rather is equivalent to

  $dbh->do("UPDATE mytable SET y = 0 WHERE x = ?", undef, $name);

Be careful not to do this:

  $dbx->do(sql[UPDATE mytable SET y = 0 WHERE x = ] . $name);

because $name as a string (rather than a reference) will be recognized
as SQL rather than a variable to interpolate.  An sql// object
concatenated with a string will append the string verbatim into your
result SQL.  Future versions of SQL::Interpolate may throw an error if
one attempts to concatenate an sql// object with a string.  It's a
better idea to do this is you must:

  $dbx->do(sql[UPDATE mytable SET y = 0 WHERE x = ] . sql[$name]);

or simply

  $dbx->do(sql[UPDATE mytable SET y = 0 WHERE x = $name]);

because then the $name value will be properly interpolated.

=head2 Source filtering

The source filtering option provides Perl an additional quote-like
operator (see L<perlop>) denoted sql//.  The quote can contain
SQL and Perl variables.

  sql/SELECT * FROM mytable WHERE x = $x/;

Source filtering will transform this construct into an object
contructor of type SQL::Interpolate::SQL with
the interpolation list passed as a parameter:

  SQL::Interpolate::SQL->new("SELECT * FROM mytable WHERE x = ", \$x);

If this object is passed to C<sql_interp> (or C<dbi_interp>), the function
will unwrap (flatten) the contained interpolation list and render it into
SQL ($sql) and bind values (@bind) as usual:

  "SELECT * FROM mytable WHERE x = ?", ($x)

To enable the quote-like sql// operator, add a "FILTER => 1" to your use
statement:

  use SQL::Interpolate FILTER => 1, qw(:all);

Just as it is possible to do with q// or qq// operators, you can use
various delimiters on the sql// operator, such as

  sql[SELECT * from mytable WHERE x = $x]
  sql(SELECT * from mytable WHERE x = $x)
  sql<SELECT * from mytable WHERE x = $x>
  sql/SELECT * from mytable WHERE x = $x/

SQL:::Interpolate::SQL objects (and sql// operators representing them)
come with a string concatenation operator (.), so you can
do things like

  sql[
    SELECT partnum, desc, price, stock
    FROM inventory
  ] . $show_all ? sql[] : sql[WHERE price > $price AND stock IN $stocks]

Disclaimer:
Source filtering is somewhat experimental.  It relies on the Text::Balanced
module  to lex Perl source code, which is a difficult task.
Therefore, I won't be surprised if sql// will fail on obscure syntax.
Refer to the Limitations and Debugging sections for additional details
in effectively using source filtering.

=head2 DBIx::Interpolate

L<DBIx::Interpolate|DBIx::Interpolate> can simplify using
SQL::Interpolate and DBI together.  What DBIx::Interpolate does is
wrap DBI in order that certain methods can take a list of
interpolation expressions instead of the usual ($sql_string, \%atts,
@bind) signature.  Therefore, instead of

  $dbh->do(dbi_interp sql[DELETE FROM mytable WHERE x = $x])

you can omit the "dbi_interp" adapter function and use things like

  $dbx->do(sql[DELETE FROM mytable WHERE x = $x]);
  $dbx->do("DELETE FROM mytable WHERE x =", \$x);
  $dbx->do("DELETE FROM mytable WHERE x =", \$x, sql[OR y=$y]);

See the DBIx::Interpolate documentation for details.

=head2 Macros

The sql// operator supports the inclusion of special macro calls that
preprocess your SQL.  These calls have an appearance similar to
built-in SQL functions (e.g. ADD, COUNT, MAX, MIN, etc.), but they are
evaluated by the sql// operator rather than by the database.

Currently, the only supported such function is LIMIT, which provides a
wrapper around constructing SQL LIMIT clauses.  The syntax for LIMIT
clauses is not standard and varies by database, so using such a macro
can improve the portability of your code.

  my($sql, @bind) = sql_interp sql[
      SELECT * FROM table
      WHERE color IN @colors
            OR color IN ['green', 'red', @colors]
            AND color IN $rcolors
               AND d = $x
      LIMIT(start => $start, count => $count*2)
  ];

Note that the parameters in the macro as interpreted as arbitrary Perl
expressions.  In fact, internally, the above gets translated to

  my($sql, @bind) = sql_interp
      "SELECT * FROM table",
      "WHERE color IN ", \@colors,
            "OR color IN", ['green', 'red', @colors],
            "AND color IN ", \$rcolors,
            "AND d = ", \$x,
      limit(start => $start, count => $count*2)
  ;

Note also how the interpolation of $start is very different from that
of $x.

The "limit" function above exportable by this module and returns
a macro object as such:

  SQL::Interpolate::Limit->new(start => $start, count => $count*2)

This macro is expanded by sql_interp, and the expansion can
be dependent upon database if sql_interp is passed a database
handle as the first argument in its parameter list:

  my($sql, @bind) = sql_interp $dbh, ...

Currently, this macro generates MySQL-like LIMIT clauses for MySQL and
Postgres-like LIMIT/OFFSET clauses for everything else.  The full
peculiarities of each are not completely supported yet.  A more
generic and robust mechanism to transparently support these and other
databases is expected in the future, possibly through Terrence
Brannon's SQL::AnyDBD module.

See the below make_sql_interp and make_dbi_intep function calls
for information on using a closure to eliminate always passing a
database handle to obtain DB-specific SQL output.

=head2 Automatic table linking

The macro processor can simplify table joins by
aliasing your tables and keys using special naming conventions and
by writing the SQL expressions that link tables together via keys.

This allows one to write

 SELECT * FROM R(S), R(Sp), R(p)
 WHERE LINK(S,Sp,p) AND S = 123

instead of

 SELECT *
 FROM  sales_order        as S,
       sales_order_line   as Sp,
       part               as p
 WHERE S.so_nbr = Sp.so_nbr AND Sp.part_nbr = p.part_nbr AND
       S.so_nbr = 123

The above example prints part information for all the line items on
sales order #123.

The table naming convention is that the names resemble the entities
related by the tables, and this is a natural way to write queries.  In
the above example the entities are sales order (S) and part (p), which
are related by the table represented by the juxtaposition "Sp".
The LINK(S,Sp,p) macro expands by equating the keys in the given relations
that identity the same entities according to the naming convention:

 "S" and "Sp" share S --> "S.so_nbr = Sp.so_nbr"
 "Sp" and "p" share p --> "Sp.part_nbr = p.part_nbr"

Also, any entity written alone in an expression (e.g. "S" in "S = 123" above)
is considered to be shorthand for primary key in a relation representing
that entity:

 "S = 123" --> "S.so_nbr = 123"

Now, in order to do all this, SQL::Interpolate needs some information
about the tables.  This is provided with the C<relations> method:

 my $interp = new SQL::Interpolate();
 $interp->relations({
     sales_order      => {name => qr/([S-T])/, key => ['so_nbr']},
     part             => {name => qr/([p-r])/, key => ['part_nbr']},
     sales_order_line => {name => qr/([S-T])([p-r])/,
                          key => ['so_nbr', 'part_nbr']}
 });

The above describes how the naming conventions map to your database
schema.  The meaning of the above is as follows.  sales order
entities can be represented by the names "S" and "T".  Part entities
can be represented by the names "p", "q", and "r".  The juxtaposition
of one of each entity name represents a row in the sales_order_line
relation that relates these two entities.  The entity names in the
matches must be surrounded by capturing parenthesis ().  The names in
the key list correspond respectively to the entities captured in the
match, and these names are the names of the primary or foreign keys
(in the current relation) that identify the entities.  For example,
"Sp" represents a row in the sales_order_line table, with represents a
relationship between a sales order entity (S) and part entity (p).
The key for S is given by Sp.so_nbr, and the key for p is given by
Sp.part_nbr.

It is possible to use two entities in the same class:

 SELECT * FROM R(Sp), R(Tp) WHERE LINK(Sp,Tp)

Also, say the product structure is represented in the database,
where parts can contain (subcomponent) parts, which can in turn contain
other (subcomponent) parts.

 $interp->relations(...
     part_part => {
         name => qr/([p-r])([p-r])/, key => ['part_nbr1', 'part_nbr2']}
 )

Now, it is possible to use two entities in the same class in the same
relation:

 SELECT * FROM R(pq), R(qs) WHERE LINK(pq,qs)

Table linking is not limited to SELECT statements either:

 UPDATE R(Sp), R(p)
 SET p.color = 'blue'
 WHERE LINK(Sp,p) AND S = $sonbr

=head2 Interpolation Examples

Results of dbi_interp are given below.

=head3 General clauses

 INPUT:  "WHERE one=", $x
 INPUT:  sql[WHERE one=$x]   (source filtering)
 OUTPUT: # if $x is a scalarref, e.g. \1
         "WHERE one=?", undef, $$x

 INPUT:  "WHERE", $x
 INPUT:  sql[WHERE $x]
 OUTPUT: # if $x is a hashref, e.g. {one => 1, two => 2}
         "WHERE x=? AND y=?", undef, $x->{one}, $x->{two}

=head3 INSERT statements

 INPUT:  "INSERT INTO mytable", $x
 INPUT:  sql[INSERT INTO mytable $x]
 OUTPUT: # if $x is a hashref, e.g. {one => 1, two => 2}
         "INSERT INTO mytable (one, two) VALUES(?, ?)", undef,
             $x->{one}, $x->{two}
 OUTPUT: # if $x is an arrayref, e.g. [1, 2]
         "INSERT INTO mytable VALUES(?, ?)", undef, @$x;
 OUTPUT: # if $x is a scalarref, e.g. \1
         "INSERT INTO mytable VALUES(?)", undef, $$x;

         # if @x = (1, 2)
 INPUT:  sql[INSERT INTO mytable @x]
 INPUT:  sql[INSERT INTO mytable [1, 2]]
 OUTPUT: "INSERT INTO mytable (?, ?)", undef \@x

         # if %x = (one => 1, two => 2)
 INPUT   sql[INSERT INTO mytable %x]
 INPUT:  sql[INSERT INTO mytable {one => 1, two => 2}]
 OUTPUT: "INSERT INTO mytable (one, two) VALUES(?, ?)", undef, \%x

=head3 IN clause

 INPUT:  "WHERE x IN", $x
 OUTPUT: # if $x is an arrayref, e.g. [1, 2]
         "WHERE x IN (?, ?)", undef, @$x
 OUTPUT: # if $x is an arrayref of length 0, e.g. []
         "WHERE NULL", undef, @$x
 OUTPUT: # if $x is a scalarref, e.g. \1
         "WHERE x in (?)", undef, $$x

The special handling for arrayref of length 0 is required because
databases (e.g. MySQL) consider "WHERE x in ()" to be invalid SQL.

=head2 Debugging

=over 4

=item TRACE_SQL

To enable tracing on C<sql_interp>, do

 use SQL::Interpolate TRACE_SQL => 1;

The generated SQL statements and bind values of all
C<sql_interp> calls will be sent to STDERR.

 DEBUG:interp[sql=INSERT INTO mytable VALUES(?),bind=5]

=item TRACE_FILTER

To enable tracing on the source code filter, do

 use SQL::Interpolate TRACE_FILTER => 1, FILTER => 1;

The source code of the module after source filtering will be sent
to STDERR.

 ...
 SQL::Interpolate::SQL->new(qq[SELECT * FROM mytable WHERE x = ], \$x)
 ...

=back

=head1 Reference

=head2 Public Functions

=over 4

=item C<sql_interp>

  ($sql, @bind) = sql_interp @params;            # functional
  ($sql, @bind) = $interp->sql_interp(@params);  # OO

Strings together the given list of interpolation parameters to return
properly formatted SQL string containing bind ("?") parameters
along with the corresponding list of bind values.

The "interpolation list" can contain strings of SQL, references to
variables to be interpolated, macros
(i.e. SQL::Interpolate::Macro-derived objects), and sql// objects
(i.e. source filtering-derived SQL::Interpolate::SQL objects, each of
which recursively contains an interpolation list.  The first parameter
may optionally be a database handle, in which case sql_interp may
generate SQL specific to that SQL dialect (e.g. LIMIT clauses).

Any sql// objects and macros passed in are first flattened
(via the sql_flatten function).  Then strings are
copied into the output SQL (possibly with some content-dependent tweaking),
while the references are dereferenced and interpolated into
the SQL using binding ("?") placeholders, and the corresponding values
pushed into @bind.

Here is an example of invoking sql_interp:

  my @colors = ('blue', 'green');
  my($sql, @bind) = sql_interp
      "SELECT * FROM table WHERE color IN", \@colors,
      "AND y =", \$x
  ;

which results in the following:

  $sql = "SELECT * FROM table WHERE color IN (?, ?) AND y = ?";
  @bind = ('blue', 'green', $x);

Here's an example using the sql// operator from source filtering:

  my $column_name = "area";
  my($sql, @bind) = sql_interp
      sql[SELECT * FROM table WHERE color IN @colors],
      "AND $column_name =", \$a
  ;

which by source filtering internally gets translated to

  my($sql, @bind) = sql_interp
      SQL::Interpolate::SQL->new("SELECT * FROM table WHERE color IN ",
                                 \@colors),
      "AND area =", \$a
  ;

sql_interp will flatten this and produces the same result as the above.

Note that variables must be passed as references (e.g. \$x) or using
using the sql// operator, otherwise they will recognized as SQL
fragments to be interpolated verbatim into the result SQL string.
Also, whitespace is automatically added between parameters, so you
don't need to insert a extra spaces:

  sql_interp "UPDATE mytable SET", {x => 2}, "WHERE y IN", \@colors

gives the SQL

  "UPDATE mytable SET x = ? WHERE y in (?, ?)"

not

  "UPDATE mytable SETx = 2WHERE y IN(?, ?)"

sql_interp will Do The Right Thing(TM) on trivial cases.  For
example, the above example will generate the following SQL if @colors
is an empty list:

  SELECT * FROM table WHERE shape = 'square' AND NULL

because the following mechanical answer is not valid SQL:

  SELECT * FROM table WHERE shape = 'square' AND color IN ()

SQL::Interpolate does not attempt to further optimize away such NULL
expressions since we can just as well let the database do this, as
databases are designed to do query optimization, without loss of
generality.

Furthermore, variable interpolation is context-sensitive.  So,

  sql_interp qq[INSERT INTO mytable ], {color => 'blue', shape => 'square'};

generates

  INSERT INTO mytable (color, shape) VALUES(?, ?)

whereas the following containing the same hash expression

  sql_interp qq[UPDATE mytable SET ], {color => 'blue', shape => 'square'};

generates SQL containing a different corresponding sub-expression:

  UPDATE mytable SET color = ?, shape = ?

The OO interface shown above is useful if you need to configure the
behavior of the interpolation, such as by tailoring it to a certain
database handle (e.g. LIMIT(...) macro) or using consulting table
information for use in the R(...) and LINK(...) macros.

=item C<dbi_interp>

This is a wrapper function around sql_interp() that serves as an
adapter that returns also the \%attr value (and and sometimes
$key_field) value for using the result directly in the DBI functions.

  ($sql, $attr, @bind) = dbi_interp(@interp_list);
  ($sql, $key_field, $attr, @bind) = dbi_interp(@interp_list);

The @interp_list can also contain the results of the C<limit> and C<select_key>
functions.  These affect the \%attr and $key_field values returned.

 $dbh->selectall_hashref(dbi_interp
    "SELECT * FROM mytable WHERE x=", \$x, select_key("y")
 );

dbi_interp is unnecessary if using DBIx::Interpolate since it internally
calls dbi_interp.

 $dbx->selectall_hashref("SELECT * FROM mytable WHERE x=",
     \$x, select_key("y"));

=item C<limit>

  $limit = limit(start => $start, count => $count)

Create a LIMIT clause macro object (SQL::Interpolate::Limit, which inherits
from SQL::Interpolate::Macro) suitable for inserting into an interpolation
list.

  sql_interp q[SELECT * FROM mytable],
             limit(start => $start, count => $count)

$start is the index of the first row to return (1-indexed).
The default is 1 if undef.

$count is the maximum number of rows to return. If undef, returns all remaining
rows.

This limit object can also be generated with the source-filtering macro

  sql[LIMIT(start => $start, count => $count)]

Note: the LIMIT clause is database dependent, and SQL::Interpolate might not
work for your database without adjustment.  You can always write the limit manually
or submit a patch.

FIX: possibly use Terrence Brannon's SQL::AnyDBD to make database-independent.

=item C<select_key>

  $keyobj = select_key($key_field);

Creates and returns an SQL::Interpolate::Key object, which if
processed by dbi_interp will cause dbi_interp to return an extra
$key_field value in the result so that it is suitable for passing into
$dbh->fetchrow_hashref and related methods.

  my($sql, $key, $attr, @bind) =
  my @params = dbi_interp q[SELECT * FROM mytable], &select_key('itemid');
  $dbh->selectall_hashref(@params);

=item C<attr>

  $attrobj = attr(%attr);

Creates and returns an SQL::Interpolate::Attr object, which if
processed by dbi_interp will cause dbi_interp to add the provided
key-value pair to the $attr hashref used by DBI methods.

  my($sql, $attr, @bind) =
  my @params =
    dbi_interp q[SELECT a, b FROM mytable], &attr(Columns=>[1,2]);
  $dbh->selectcol_arrayref(@params);

=item C<make_sql_interp>

  my $sql_interp = make_sql_interp(@params);          # functional
  my $sql_interp = $interp->make_sql_interp(@params); # OO

This creates a closure that wraps the sql_interp function such that
the parameters passed to the sql_interp consist of @params followed by
the parameters passed to the closure.  This function is typically used
to eliminate a need to always pass in a database handle into
sql_interp:

  my $interp = make_sql_interp($dbh);

  my($sql, @bind) = $interp->(sql[
     SELECT * FROM x WHERE y > $z
     LIMIT(start => $start, count => $count)
  ]);

=item C<make_dbi_interp>

  $dbi_interp = make_dbi_interp(@params);          # functional
  $dbi_interp = $interp->make_dbi_interp(@params); # OO

This is similar in make_sql_interp except that is generates
a closure around the dbi_interp function rather than the sql_interp
function.

=back

=head2 Private Functions

=over 4

=item C<sql_flatten>

 @list_out = sql_flatten(@list_in);          # functional
 @list_out = $interp->sql_flatten(@list_in); # OO

Flatten out an interpolation list and expands any macros.

 my @list = sql_flatten sql[SELECT * FROM mytable where x=$x]
 # list now contains ('SELECT * FROM mytable where x=', \$x).

This function is typically used internally by SQL::Interpolate,
particularly by sql_interp and dbi_interp.

This function takes the same type of input as sql_interp and in
fact sql_interp uses it to preprocess input.  The result of
this function is a list of SQL strings and variable references for
interpolation.

=back

=head2 Public Methods

=over 4

=item C<new>

Creates a new SQL::Interpolate object.

 my $interp = new SQL::Interpolate(dbh => $dbh, relations => $relations);

$dbh [optional] is a DBI database handle.

$relations [optional] is hashref of relations as passed to the
C<relations> method.

Creating an object (to use the OO interface rather than the functional
interface) is only necessary if you want to store state, such as a DBI
handle or relation information for the LINK macro and such.  Example:

 # functional interface
 @result = sql_interp @params;
 # OO interface
 @result = $interp->sql_interp(@params);

=item C<relations>

Sets information on relations (tables) for this objects.

 $interp->relations({
    messageset => {name => qr/[A-Z]/, key => 'msid'},
    message    => {name => qr/[m-p]/, key => 'mid'},
    messageset_message => {foreign => {
       msid => 'messageset',
       mid  => 'message'
    }},
    messageset_messageset => {foreign => {
       msid_1 => 'messageset',
       msid_2 => 'messageset'
    }},
 });

FIX--more documentation

=back

=head2 Private Methods

=over 4

=item C<filter_sql>

SQL preprocessor to simplify table joins.  Typically only used
internally by SQL::Interpolate.

 $interp->relations(...);
 my $sql_out = $interp->filter_sql($sql_in);

Expands simplified SQL such as

    SELECT * FROM R(Am), R(m)
    WHERE  A = ? AND LINK(Am, m)

into standard SQL:

    SELECT * FROM messageset_message as Am, message as m
    WHERE  Am.msid = ? AND Am.mid = m.mid

The SQL may contain these macros:

  R(...) - identify relation and entities on it.

  LINK(X,Y,Z,...) forms a Boolean expression linking the provided
    relations.  There must be at least one parameter (typically two).

 $sql_in : string - input string

 $sql_out : string - output string

Possible improvements:

 - support multi-part keys?
 - support optional automatic inclusion of LINK(...).

=back

=head1 STATUS / LIMITATIONS

This module is fairly new and under development, so interfaces could
change.  Robustness, good style, and good documentation are design
goals.  Still, the approach of this module does not guarantee that the
generated SQL is valid for your particular database.  SQL::Interpolate
does not parse most of the SQL input but just passes it through.  So,
it is really the responsibility of the database to validate your SQL.
On the positive side, this module has no problem with many
non-standard SQL constructs.  If in doubt, check the output of
dbi_interp yourself by enabling the TRACE_SQL option.

Source filtering, if enabled, also has the potentiel to give
unexpected results.  The module relies on Text::Balanced to lex Perl
source code.  Even though Text::Balanced makes a valiant effort, the
task is difficult and the results not always precise, especially for
very obscure Perl constructs.  If in doubt, check the output yourself
by enabling the TRACE_FILTER option.

Possible future additions include

=over 4

=item

Allow sql// operator to perform syntax checking on SQL
  (like a strict-mode). (unlikely)

=item

Allow DB-independent comments inside SQL.  These will get parsed
  out before sending to the database driver.

=item

Add more macros

=back

=head1 SEE ALSO

L<DBIx::Interpolate|DBIx::Interpolate>,
L<DBI|DBI>,
L<SQL::Abstract|SQL::Abstract>,
L<DBIx::Abstract|DBIx::Abstract>,
L<SQL::AnyDBD|SQL::AnyDBD>.

For a real example of using SQL::Interpolate, see
Meset::MessageBoard in Meset (L<http://math2.org/meset>).

=head1 CREDITS

Feedback incorporated from Mark Stosberg and Terrence Brannon.

=head1 AUTHOR

David Manura (L<http://math2.org/david/contact>).

=head1 BUGS

Bug reports and comments on the design are most welcome.
rt.cpan.org is preferred for bug reporting, but you may also
contact me directly (L<http://math2.org/david/contact>).

=head1 PROJECT PAGE

This project is currently hosted at
L<http://math2.org/sql_interpolate> .

=head1 COPYRIGHT

Copyright (c) 2003-2005, David Manura.  All Rights Reserved.
This module is free software. It may be used, redistributed
and/or modified under the same terms as Perl itself.

=cut

